package de.jstacs.algorithms.alignment;

import java.util.ArrayList;
import java.util.Arrays;

import de.jstacs.algorithms.alignment.Alignment.AlignmentType;
import de.jstacs.algorithms.alignment.cost.Costs;
import de.jstacs.data.AlphabetContainer;
import de.jstacs.data.DataSet;
import de.jstacs.data.WrongAlphabetException;
import de.jstacs.data.sequences.Sequence;
import de.jstacs.results.CategoricalResult;
import de.jstacs.utils.DoubleList;
import de.jstacs.utils.IntList;

/**
 * 
 * @author Jens Keilwagen
 */
public class CenterStarAlignment {

	private Alignment a;
	private ArrayList<ArrayList<PairwiseStringAlignment>> align;
	private ArrayList<Sequence> sequences;
	private DoubleList w;
	private AlignmentType type;
	
	public CenterStarAlignment( Costs c, AlignmentType type ) {
		this( new Alignment(c), type );
	}
	
	public CenterStarAlignment( Alignment a, AlignmentType type ) {
		this.a = a;
		w = new DoubleList();
		sequences = new ArrayList<Sequence>();
		align = new ArrayList<ArrayList<PairwiseStringAlignment>>();
		this.type=type;
	}
	
	public int getRepresentativeIndex() {
		int best = -1, n = sequences.size();
		if( n == 0 ) {
			return -1;
		}
		double bestVal = Double.POSITIVE_INFINITY, current;
		ArrayList<PairwiseStringAlignment> list;
		for( int s1 = 0; s1 <n; s1++ ) {
			current = 0;
			list = align.get(s1);
			for( int s2 = 0; s2 < n; s2++ ) {
				if( list.get(s2) != null ) {
					if( w != null ) {
						current+= w.get(s1)*w.get(s2)*list.get(s2).getCost();
					} else {
						current+= list.get(s2).getCost();
					}
				}
			}
			if( current < bestVal ) {
				bestVal = current;
				best = s1;
			}
		}
		return best;
	}
	
	public Sequence getRepresentative() {
		int idx = getRepresentativeIndex();
		if( idx >= 0 ) {
			return sequences.get(idx);
		} else {
			return null;
		}
	}
	
	private boolean isSymmetric() {
		return type == AlignmentType.GLOBAL || type == AlignmentType.FREE_SHIFT;
	}
	
	public void add( Sequence seq, double w ) {
		ArrayList<PairwiseStringAlignment> current = new ArrayList<PairwiseStringAlignment>();
		PairwiseStringAlignment pa;
		boolean symmetric = isSymmetric();
		for( int i = 0; i < sequences.size(); i++ ) {
			pa = a.getAlignment( type, sequences.get(i), seq );
			align.get(i).add(pa);
			if( symmetric ) {
				current.add(pa);
			} else {
				current.add( a.getAlignment( type, seq, sequences.get(i) ) );
			}
		}
		current.add(null);
		sequences.add(seq);
		align.add(current);
		this.w.add(w);
	}
	
	public StringAlignment getAlignment() {
		boolean symmeteric = isSymmetric();
		int best = getRepresentativeIndex();
		return getAlignment( best, align.get( best ), symmeteric );
	}
	
	public static StringAlignment getAlignmentWithReference( int referenceIdx, DataSet data, Alignment a, AlignmentType type ) {
		ArrayList<PairwiseStringAlignment> list = new ArrayList<PairwiseStringAlignment>();
		Sequence ref = data.getElementAt( referenceIdx );
		for( int i = 0; i < data.getNumberOfElements(); i++ ) {
			if( i == referenceIdx ) {
				list.add( null );
			} else {
				list.add( a.getAlignment( type, ref, data.getElementAt(i) ) );
			}
		}
		return getAlignment( referenceIdx, list, false );
	}
	
	private static StringAlignment getAlignment( int best, ArrayList<PairwiseStringAlignment> list, boolean symmetric ) {
		//prepare
		StringBuffer[] help = new StringBuffer[list.size()];
		boolean[] used = new boolean[help.length];
		Arrays.fill(used, false);
		used[best] = true;
				
		//build alignment
		int i1, i2, posAll, l, next;
		double bestVal;
		PairwiseStringAlignment pa;
		IntList order = new IntList();
		order.add( best );
		for( int idx = 1; idx < help.length; idx++ ) {
			//find best remaining sequence
			bestVal = Double.POSITIVE_INFINITY;
			next = -1;
			for( int s2 = 0; s2 < help.length; s2++ ) {
				if( !used[s2] && list.get(s2).getCost() < bestVal ) {
					bestVal = list.get(s2).getCost();
					next = s2;
				}
			}
			
			if( !symmetric || best < next ) {
				i1 = 0;
				i2 = 1;
			} else {
				i1 = 1;
				i2 = 0;
			}
			order.add( next );
			pa = list.get(next);
			String h1 = pa.getAlignedString(i1), h2 = pa.getAlignedString(i2);
			if( idx == 1 ) {
				help[best] = new StringBuffer( h1 );
				help[next] = new StringBuffer( h2 );
			} else {
				help[next] = new StringBuffer();
				for( posAll = 0, l = 0; l < h1.length(); l++ ) {
					//System.out.println( l + "\t" + posAll + "\t" + help[best].length() );
					if( h1.charAt(l) == '-' ) { //gap in center sequence in pairwise alignment
						if( help[best].length() == posAll || help[best].charAt( posAll ) != '-' ) {
							for( int s1 = 0; s1 < help.length; s1++ ) {
								if( s1 != next && help[s1]!= null ) {
									help[s1].insert(posAll, '-');
								}
							}
						}
					} else { //match, mismatch or gap in second sequence (next) of pairwise alignment
						//skip gaps in already align sequences 
						while( help[best].charAt(posAll) == '-' ) {
							 posAll++;
							 help[next].append('-');
						}
					}
					posAll++;
					help[next].append( h2.charAt(l) );
				}
				for( ; posAll < help[best].length(); posAll++ ) {
					help[next].append('-');
				}
			}
			
			used[next]=true;
		}
		
		String[] seqs = new String[help.length];
		for( int i = 0; i < help.length; i++ ) {
			seqs[i] = help[i].toString();
		}
		
		//create StringAlignment
		return new StringAlignment(Double.NaN/**TODO*/, seqs, new CategoricalResult("order", "the order of the sequences", order.toString() ) );
	}
	
	/*
	public static StringAlignment refine( Alignment a, AlphabetContainer con, StringAlignment align, double percentage, int window ) throws IllegalArgumentException, WrongAlphabetException {
		int n = align.getNumberOfAlignedSequences();
		String[] seqs = new String[n], res;
		for( int i = 0; i < n; i++ ) {
			seqs[i] = align.getAlignedString( i );
		}
		res = seqs.clone();
		
		//determine gaps
		double gaps;
		boolean[] refine = new boolean[align.getLength()];
		Arrays.fill( refine, false );
		for( int l = 0; l < refine.length; l++ ) {
			gaps = 0;
			for( int i = 0; i < n; i++ ) {
				if( seqs[i].charAt(l) == '-' ) {
					gaps++;
				}
			}
			refine[l] = gaps / n > percentage;
		}
		
		//
		int next = 0;
		for( int l = 0; l < refine.length; l++ ) {
			if( refine[l] ) {
				for( int k = Math.max(0,l-window); k < l; k++ ) {
					refine[k] = true;
				}
				next = window;
			} else {
				if( next > 0 ) {
					refine[l] = true;
					next--;
				}
			}
		}

		CenterStarAlignment c = new CenterStarAlignment( a );
		int m = 0;
		for( int l = 0; l < refine.length; l++ ) {
			if( refine[l] ) {
				int startIdx=l++;
				while( l < refine.length && refine[l] ) {
					l++;
				}
				c.sequences.clear();
				c.align.clear();
				c.w.clear();
				int idx = -1, w = 0;
				for( int i = 0; i < n; i++ ) {
					String residues = seqs[i].substring( startIdx, l).replaceAll("-","");
					c.add( Sequence.create( con, residues ), 1 );
					if( residues.length() > w ) {
						w = residues.length();
						idx = i;
					}
				}
				StringAlignment sa = 
					c.getAlignment();
					//getAlignment( idx, c.align.get( idx ), a.type == AlignmentType.GLOBAL );
				
				//System.out.println( sa.getAnnotationResult().getValue() );
				int end = m+(l-startIdx);
				for( int i = 0; i < n; i++ ) {
					res[i] = res[i].substring(0,m) + sa.getAlignedString(i) + res[i].substring(end); 
				}
				m = m + sa.getLength()+1;
			} else {
				m++;
			}
		}

		return new StringAlignment(Double.NaN, res, align.getAnnotationResult() );
	}
	/**/
}
